@@ -36,10 +36,13 @@ class AsyncSearchClient(NamespacedClient):
         pretty: t.Optional[bool] = None,
     ) -> ObjectApiResponse[t.Any]:
         """
-        Deletes an async search by ID. If the search is still running, the search request
-        will be cancelled. Otherwise, the saved search results are deleted.
+        Deletes an async search by identifier. If the search is still running, the search
+        request will be cancelled. Otherwise, the saved search results are deleted. If
+        the Elasticsearch security features are enabled, the deletion of a specific async
+        search is restricted to: the authenticated user that submitted the original search
+        request; users that have the `cancel_task` cluster privilege.
 
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/8.13/async-search.html>`_
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/8.15/async-search.html>`_
 
         :param id: A unique identifier for the async search.
         """
@@ -74,18 +77,20 @@ class AsyncSearchClient(NamespacedClient):
         error_trace: t.Optional[bool] = None,
         filter_path: t.Optional[t.Union[str, t.Sequence[str]]] = None,
         human: t.Optional[bool] = None,
-        keep_alive: t.Optional[t.Union["t.Literal[-1]", "t.Literal[0]", str]] = None,
+        keep_alive: t.Optional[t.Union[str, t.Literal[-1], t.Literal[0]]] = None,
         pretty: t.Optional[bool] = None,
         typed_keys: t.Optional[bool] = None,
         wait_for_completion_timeout: t.Optional[
-            t.Union["t.Literal[-1]", "t.Literal[0]", str]
+            t.Union[str, t.Literal[-1], t.Literal[0]]
         ] = None,
     ) -> ObjectApiResponse[t.Any]:
         """
         Retrieves the results of a previously submitted async search request given its
-        ID.
+        identifier. If the Elasticsearch security features are enabled, access to the
+        results of a specific async search is restricted to the user or API key that
+        submitted it.
 
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/8.13/async-search.html>`_
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/8.15/async-search.html>`_
 
         :param id: A unique identifier for the async search.
         :param keep_alive: Specifies how long the async search should be available in
@@ -143,10 +148,12 @@ class AsyncSearchClient(NamespacedClient):
         pretty: t.Optional[bool] = None,
     ) -> ObjectApiResponse[t.Any]:
         """
-        Retrieves the status of a previously submitted async search request given its
-        ID.
+        Get async search status Retrieves the status of a previously submitted async
+        search request given its identifier, without retrieving search results. If the
+        Elasticsearch security features are enabled, use of this API is restricted to
+        the `monitoring_user` role.
 
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/8.13/async-search.html>`_
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/8.15/async-search.html>`_
 
         :param id: A unique identifier for the async search.
         """
@@ -229,16 +236,16 @@ class AsyncSearchClient(NamespacedClient):
         batched_reduce_size: t.Optional[int] = None,
         ccs_minimize_roundtrips: t.Optional[bool] = None,
         collapse: t.Optional[t.Mapping[str, t.Any]] = None,
-        default_operator: t.Optional[t.Union["t.Literal['and', 'or']", str]] = None,
+        default_operator: t.Optional[t.Union[str, t.Literal["and", "or"]]] = None,
         df: t.Optional[str] = None,
         docvalue_fields: t.Optional[t.Sequence[t.Mapping[str, t.Any]]] = None,
         error_trace: t.Optional[bool] = None,
         expand_wildcards: t.Optional[
             t.Union[
                 t.Sequence[
-                    t.Union["t.Literal['all', 'closed', 'hidden', 'none', 'open']", str]
+                    t.Union[str, t.Literal["all", "closed", "hidden", "none", "open"]]
                 ],
-                t.Union["t.Literal['all', 'closed', 'hidden', 'none', 'open']", str],
+                t.Union[str, t.Literal["all", "closed", "hidden", "none", "open"]],
             ]
         ] = None,
         explain: t.Optional[bool] = None,
@@ -251,7 +258,7 @@ class AsyncSearchClient(NamespacedClient):
         ignore_throttled: t.Optional[bool] = None,
         ignore_unavailable: t.Optional[bool] = None,
         indices_boost: t.Optional[t.Sequence[t.Mapping[str, float]]] = None,
-        keep_alive: t.Optional[t.Union["t.Literal[-1]", "t.Literal[0]", str]] = None,
+        keep_alive: t.Optional[t.Union[str, t.Literal[-1], t.Literal[0]]] = None,
         keep_on_completion: t.Optional[bool] = None,
         knn: t.Optional[
             t.Union[t.Mapping[str, t.Any], t.Sequence[t.Mapping[str, t.Any]]]
@@ -276,12 +283,12 @@ class AsyncSearchClient(NamespacedClient):
         routing: t.Optional[str] = None,
         runtime_mappings: t.Optional[t.Mapping[str, t.Mapping[str, t.Any]]] = None,
         script_fields: t.Optional[t.Mapping[str, t.Mapping[str, t.Any]]] = None,
-        scroll: t.Optional[t.Union["t.Literal[-1]", "t.Literal[0]", str]] = None,
+        scroll: t.Optional[t.Union[str, t.Literal[-1], t.Literal[0]]] = None,
         search_after: t.Optional[
             t.Sequence[t.Union[None, bool, float, int, str, t.Any]]
         ] = None,
         search_type: t.Optional[
-            t.Union["t.Literal['dfs_query_then_fetch', 'query_then_fetch']", str]
+            t.Union[str, t.Literal["dfs_query_then_fetch", "query_then_fetch"]]
         ] = None,
         seq_no_primary_term: t.Optional[bool] = None,
         size: t.Optional[int] = None,
@@ -300,7 +307,7 @@ class AsyncSearchClient(NamespacedClient):
         suggest: t.Optional[t.Mapping[str, t.Any]] = None,
         suggest_field: t.Optional[str] = None,
         suggest_mode: t.Optional[
-            t.Union["t.Literal['always', 'missing', 'popular']", str]
+            t.Union[str, t.Literal["always", "missing", "popular"]]
         ] = None,
         suggest_size: t.Optional[int] = None,
         suggest_text: t.Optional[str] = None,
@@ -311,14 +318,22 @@ class AsyncSearchClient(NamespacedClient):
         typed_keys: t.Optional[bool] = None,
         version: t.Optional[bool] = None,
         wait_for_completion_timeout: t.Optional[
-            t.Union["t.Literal[-1]", "t.Literal[0]", str]
+            t.Union[str, t.Literal[-1], t.Literal[0]]
         ] = None,
         body: t.Optional[t.Dict[str, t.Any]] = None,
     ) -> ObjectApiResponse[t.Any]:
         """
-        Executes a search request asynchronously.
+        Runs a search request asynchronously. When the primary sort of the results is
+        an indexed field, shards get sorted based on minimum and maximum value that they
+        hold for that field, hence partial results become available following the sort
+        criteria that was requested. Warning: Async search does not support scroll nor
+        search requests that only include the suggest section. By default, Elasticsearch
+        doesn’t allow you to store an async search response larger than 10Mb and an attempt
+        to do this results in an error. The maximum allowed size for a stored async search
+        response can be set by changing the `search.max_async_search_response_size` cluster
+        level setting.
 
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/8.13/async-search.html>`_
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/8.15/async-search.html>`_
 
         :param index: A comma-separated list of index names to search; use `_all` or
             empty string to perform the operation on all indices
